#pragma once

#include <cemapi.h>
#include <mapiutil.h>
#include <string>
#include <vector>

namespace MAPI {

class CMAPI;
class CBase;
class CSession;
class CMsgStore;
class CFolder;
class CMessage;

typedef std::vector<CMsgStore*> CMsgStores;
typedef std::vector<CFolder*> CFolders;
typedef std::vector<CMessage*> CMessages;

//[eBeB̊֐Q
BOOL QueryRows(IMAPITable *pTable, LPSRowSet *lppRows);
template <class T> void Release(std::vector<T> *pVector);

/**
 * MAPI{NX
 */
class CMAPI
{
public:
	CMAPI(void);
	~CMAPI(void);

private:
	BOOL m_AlreadyInitialized;
};

/**
 * BaseNX
 */
class CBase
{
public:
	virtual std::wstring GetDisplayName(void);
	virtual BOOL GetProperty(ULONG ulPropTag, LPSPropValue *lppProp);
	virtual BOOL SetProperty(LPSPropValue lpProp);

protected:
	virtual std::wstring GetStringProperty(ULONG ulPropTag);
	virtual void GetSystimeProperty(ULONG ulPropTag, LPSYSTEMTIME lpSysTime);
	virtual BOOL GetEntryId(LPSPropValue *lppProp);

	virtual LPMAPIPROP GetMAPIProp(void) = 0;
};

/**
 * SessionNX
 */
class CSession
{
public:
	CSession(void);
	~CSession(void);

	CMsgStores &GetAccounts(void);
	CMsgStore &GetAccount(const std::wstring &AccountName);
	CMsgStore &GetPdxAccount(void);

	CFolder &GetFolder(const std::wstring &FolderPath);
	CFolder &GetPdxRecievedFolder(void);

private:
	typedef std::vector<std::wstring> Strings;

	void Release(void);
	BOOL IsPdxAccountName(const std::wstring &AccountName);
	Strings SplitPath(const std::wstring &FolderPath);

private:
	IMAPISession *m_pSession;
	IMAPITable *m_pMsgStoresTable;

	CMsgStores *m_pAccounts;
};

/**
 * MsgStoreNX
 */
class CMsgStore : public CBase
{
public:
	CMsgStore(IMAPISession *pSession, SBinary entry);
	~CMsgStore(void);

	CFolder &GetRootFolder(void);
	CFolder &GetInboxFolder(void);

protected:
	virtual inline LPMAPIPROP GetMAPIProp(void) { return m_pMsgStore; };

private:
	void Release(void);
	CFolder &GetFolder(ULONG ulPropTag, CFolder **ppFolder);

private:
	IMAPISession *m_pSession;
	LPMDB m_pMsgStore;

	CFolder *m_pRootFolder;
	CFolder *m_pInboxFolder;
};

/**
 * FolderNX
 */
class CFolder : public CBase
{
public:
	CFolder(IMAPISession *pSession, LPMDB pMsgStore, SBinary entry, const std::wstring ParentPath);
	CFolder(IMAPISession *pSession, LPMAPIFOLDER pFolder, SBinary entry, const std::wstring ParentPath);
	~CFolder(void);

	std::wstring &GetPath(void) { return m_Path; };

	CFolders &GetSubFolders(BOOL IsRefresh = FALSE);
	CFolder &GetSubFolder(const std::wstring &FolderName, BOOL IsRefresh = FALSE);

	CMessages &GetAllMessages(void);
	CMessages &GetNoreadMessages(void);
	ULONG GetAllMessageCount(void);

	void EnumMessages(BOOL (*pEnumMsgFunc)(CFolder &, LPSRow, LPVOID), LPSPropTagArray lpTags, LPVOID lpParam);

	BOOL CopyMessage(SBinary entry, CFolder &destFolder, ULONG ulFlags);

	BOOL MailSwitchToFolder(void);

protected:
	virtual inline LPMAPIPROP GetMAPIProp(void) { return m_pFolder; };

private:
	void Release(void);
	IMAPITable* GetHierarchy(void);
	IMAPITable* GetContents(void);

	static BOOL EnumMsgAllFunc(CFolder &thisFolder, LPSRow pRow, LPVOID lpParam);
	static BOOL EnumMsgNoreadFunc(CFolder &thisFolder, LPSRow pRow, LPVOID lpParam);

private:
	IMAPISession *m_pSession;
	LPMAPIFOLDER m_pFolder;
	IMAPITable *m_pHierarchy;
	IMAPITable *m_pContents;
	std::wstring m_Path;

	CFolders *m_pSubFolders;
	CMessages m_Messages;
};

/**
 * MessageNX
 */
class CMessage : public CBase
{
public:
	CMessage(IMAPISession *pSession, SBinary entry);
	~CMessage(void);

	std::wstring GetSubject(void);
	std::wstring GetSenderName(void);
	std::wstring GetSenderEmailAddress(void);
	std::wstring GetMessageClass(void);
	void GetDeliveryTime(LPSYSTEMTIME lpSysTime);
	BOOL IsRead(void);
	BOOL SetRead(BOOL flag);

	BOOL Display(void);

protected:
	virtual inline LPMAPIPROP GetMAPIProp(void) { return m_pMessage; };

private:
	void Release(void);

private:
	IMAPISession *m_pSession;
	LPMESSAGE m_pMessage;
};

}; //namespace MAPI